﻿#region License Information
/* HeuristicLab
 * Copyright (C) Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using HEAL.Attic;

namespace HeuristicLab.Problems.DataAnalysis {
  [StorableType("3574E222-1167-4F1B-93D2-01B15A7F1E84")]
  public abstract class Vector<T> : IVector<T> {
    [Storable]
    protected readonly List<T> Values;

    protected Vector(IEnumerable<T> values) {
      this.Values = values.ToList();
    }

    [StorableConstructor]
    protected Vector(StorableConstructorFlag _) { }

    public override string ToString() {
      const int maxCount = 10;
      string extension = Values.Count > maxCount ? ", ..." : "";
      return $"[{string.Join(", ", Values.Cast<object>().Take(Math.Min(Values.Count, maxCount)))}{extension}]";
    }

    #region Interface members

    public int Count {
      get { return Values.Count; }
    }

    public T this[int index] {
      get { return Values[index]; }
    }

    public IEnumerator<T> GetEnumerator() {
      return Values.GetEnumerator();
    }

    IEnumerator IEnumerable.GetEnumerator() {
      return GetEnumerator();
    }

    #endregion
  }
}